"""This file demonstrates how to create a DataTable with a dynamic first column
that expands to fill the available space, while keeping other columns at
fixed widths. The table will also auto-adjust the first column's width
when the terminal is resized.

Recipe by Edward Jazzhands"""

import sys
from typing import Any
from textual.widgets import DataTable
from textual.widgets.data_table import ColumnKey
from textual.app import App, ComposeResult
from textual.containers import Container

ROWS = [
    ("Joseph Schooling", "Singapore", 50.39, 4),
    ("Michael Phelps", "United States", 51.14, 2),
    ("Chad le Clos", "South Africa", 51.14, 5),
    ("László Cseh", "Hungary", 51.14, 6),
    ("Li Zhuhao", "China", 51.26, 3),
    ("Mehdy Metella", "France", 51.58, 8),
    ("Tom Shields", "United States", 51.73, 7),
    ("Aleksandr Sadovnikov", "Russia", 51.84, 1),
    ("Darren Burns", "Scotland", 51.84, 10),
]

class MyDataTable(DataTable[Any]):

    col1_minimum = 15   # dynamic column
    col2_width = 20
    col3_width = 15
    col4_width = 10
    other_cols_total = col2_width + col3_width + col4_width

    def on_mount(self) -> None:

        self.add_column("Name", width=self.col1_minimum, key="col1")
        self.add_column("Country", width=self.col2_width, key="col2")
        self.add_column("Time", width=self.col3_width, key="col3")
        self.add_column("Lane", width=self.col4_width, key="col4")
        self.add_rows(ROWS) 

    def on_resize(self) -> None:
        
        # Account for padding on both sides of each column:
        total_cell_padding = self.cell_padding * (len(self.columns)*2)

        # Pretty obvious how this works I think:
        first_col_width = self.size.width - self.other_cols_total - total_cell_padding

        # Prevent column from being smaller than the chosen minimum:
        if first_col_width < self.col1_minimum:
            first_col_width = self.col1_minimum

        self.columns[ColumnKey("col1")].width = first_col_width
        self.refresh()
        

class TextualApp(App[None]):

    def compose(self) -> ComposeResult:
        with Container():
            yield MyDataTable()


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This file demonstrates the usage of Textual's secret 'Signal' class.
This isn't a public API, but it is used internally by Textual.
It allows widgets to communicate with each other without sending events
or passing in direct references to anything.

Recipe by Edward Jazzhands"""

import sys
from textual import on
from textual.app import App, ComposeResult
from textual.signal import Signal
from textual.widget import Widget
from textual.widgets import Static, Button, Footer


class MyManager(Widget):

    def __init__(self):
        super().__init__()
        self.display = False

        self.my_special_num = 42
        self.my_signal: Signal[int] = Signal(self, "my_signal")

    def send_signal(self):
        self.my_signal.publish(self.my_special_num)


class TextualApp(App[None]):

    CSS = """
    Screen { align: center middle; }
    #my_static { border: solid blue; width: auto;}
    """

    BINDINGS = [
        ("b", "send_signal", "Send Signal"),
    ]

    def __init__(self):
        super().__init__()
        self.manager = MyManager()

    def compose(self) -> ComposeResult:
        self.manager = MyManager()
        yield self.manager

        yield Static("Press button or press b", id="my_static")
        yield Button("Send Signal", id="send_signal_button")
        yield Footer()

    def on_mount(self) -> None:
        self.manager.my_signal.subscribe(self, self.on_my_signal)

    @on(Button.Pressed, "#send_signal_button")
    def action_send_signal(self):
        self.manager.send_signal()

    def on_my_signal(self, value: int):
        self.notify(f"Received signal with value: {value}")


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""
This example demonstrates how to capture mouse events in Textual applications 
in order to get clean mouse capture behavior. 
This shows two different ways to capture mouse events:
    1. Using `capture_mouse` and `release_mouse` methods.
    2. Using a flag to track if the click started on the widget.
The second method can be preferable in some cases, as it avoids the need to
explicitly capture and release the mouse.
You can also see that removing either of the `on_leave` event handlers
will result in the mouse not being released when the mouse leaves the widget,
and the mouse_up event will be triggered even if the mouse is not
still over the widget.

Recipe by Edward Jazzhands"""

import sys
from typing import Any
from textual.app import App
from textual import events
from textual.widgets import Static


class MyCapturer1(Static):

    def on_mouse_down(self, event: events.MouseDown) -> None:

        self.capture_mouse()
        self.add_class("pressed")

    def on_mouse_up(self, event: events.MouseUp) -> None:

        if self.app.mouse_captured == self:
            self.release_mouse()            
            self.remove_class("pressed")
            self.notify("Mouse released")

    def on_leave(self) -> None:

        self.release_mouse()
        self.remove_class("pressed")


class MyCapturer2(Static):

    def __init__(self, *args: Any, **kwargs: Any):
        super().__init__(*args, **kwargs)
        self.click_started_on = False

    def on_mouse_down(self, event: events.MouseDown) -> None:

        self.click_started_on = True
        self.add_class("pressed")

    def on_mouse_up(self) -> None:

        self.remove_class("pressed")
        if self.click_started_on:
            self.click_started_on = False
            self.notify("Mouse released")

    def on_leave(self) -> None:

        self.click_started_on = False
        self.remove_class("pressed")


class TextualApp(App[None]):

    CSS = """
    Screen { align: center middle; }
    #capture1 { border: solid blue; }
    #capture2 { border: solid red; }
    .capture { 
        width: auto;
        &.pressed { background: $success; }
    }
    """

    def compose(self):

        yield MyCapturer1("Capturer 1", id="capture1", classes="capture")
        yield MyCapturer2("Capturer 2", id="capture2", classes="capture")


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This script demonstrates how widgets do not have a `CSS` class attribute.
It is fairly common to assume that they do, but they do not.
The code below will not work as expected.
Widgets must use the `DEFAULT_CSS` class attribute instead.

Example by Edward Jazzhands, 2025"""

from textual.app import App, ComposeResult
from textual.widget import Widget
from textual.widgets import Header, Static

class MyContainer(Widget):

    CSS = """     # Widget.CSS does not exist, use DEFAULT_CSS instead
    #porque {
        border: round red;
    }
    """

    def compose(self) -> ComposeResult:
        yield Static("Hello World", id="porque")


class BorderTestApp(App):
    CSS = """
    MyContainer {
        border: round blue;
    }
    """

    def compose(self) -> ComposeResult:
        yield Header()
        yield MyContainer()


if __name__ == "__main__":
    app = BorderTestApp()
    app.run()


"""This script demonstrates how to change the options of a Select widget in real-time.

Recipe by Edward Jazzhands"""

import sys
from textual.app import App
from textual.widgets import Select, Button
from textual.containers import Container

class TextualApp(App[None]):

    DEFAULT_CSS = """
    #my_container { width: 1fr; height: 1fr; align: center middle; }
    """

    my_list = [
        ("Option 1", "1"),
        ("Option 2", "2"),
        ("Option 3", "3"),
        ("Option 4", "4"),
        ("Option 5", "5"),
    ]
    my_list2 = [
        ("Option 6", "6"),
        ("Option 7", "7"),
        ("Option 8", "8"),
        ("Option 9", "9"),
        ("Option 10", "10"),
    ]
    
    def compose(self):

        with Container(id="my_container"):
            self.my_select = Select(self.my_list)
            yield self.my_select
            yield Button("Change list")

    def on_button_pressed(self, event: Button.Pressed):
        self.my_select.set_options(self.my_list2)


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""this file is a work in progress please ignore

Recipe by Edward Jazzhands"""

from __future__ import annotations
import sys
# import importlib.resources
from pathlib import Path

from textual.app import App, ComposeResult

from textual.screen import ModalScreen
from textual.widgets import Markdown
from textual.containers import VerticalScroll
from textual.binding import Binding
from textual.widgets import Static, Footer


class HelpScreen(ModalScreen[None]):

    BINDINGS = [
        Binding("escape,enter", "close_screen", description="Close the help window.", show=True),
    ]

    def __init__(self, anchor: str | None = None) -> None:
        super().__init__()
        self.anchor_line = anchor

    def compose(self) -> ComposeResult:

        help_path = Path(__file__).parent / "help.md"
        if not help_path.exists():
            raise FileNotFoundError(f"Help file not found at {help_path}")
        with help_path.open(encoding="utf-8") as f:
            self.help = f.read()

        with VerticalScroll(classes="screen_container help"):
            yield Markdown(self.help)

    def on_mount(self) -> None:
        self.query_one(VerticalScroll).focus()
        if self.anchor_line:
            found = self.query_one(Markdown).goto_anchor(self.anchor_line)
            if not found:
                self.log.error(f"Anchor '{self.anchor_line}' not found in help document.")

    def on_click(self) -> None:
        self.dismiss()

    def action_close_screen(self) -> None:
        self.dismiss()

    def go_to_anchor(self, anchor: str) -> None:
        """Scroll to a specific anchor in the help screen."""

class TextualApp(App[None]):

    CSS = """
    Screen { align: center middle; }
    #my_static { border: solid blue; width: auto;}
    """


    def compose(self):

        yield Static("Hello, Textual!", id="my_static")
        yield Footer()

    def action_binding(self):
        self.notify("You pressed the 'b' key!")


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This example demonstrates how to create a sortable DataTable
with headers that change their appearance based on the sorting state.
The arrows in the header label will flip direction when the user clicks
on a header to indicate the current sort order.

Recipe by Edward Jazzhands"""

# python standard lib
from __future__ import annotations
import sys
from typing import Any
from enum import Enum

# Textual imports
from textual import on
from textual.app import App, ComposeResult
from textual.widgets import DataTable
from textual.widgets.data_table import Column, ColumnKey
from textual.containers import Container
from rich.text import Text


ROWS = [
    (4, "Joseph Schooling", "Singapore", 50.39),
    (2, "Michael Phelps", "United States", 51.14),
    (5, "Chad le Clos", "South Africa", 51.14),
    (6, "László Cseh", "Hungary", 51.14),
    (3, "Li Zhuhao", "China", 51.26),
    (8, "Mehdy Metella", "France", 51.58),
    (7, "Tom Shields", "United States", 51.73),
    (1, "Aleksandr Sadovnikov", "Russia", 51.84),
    (10, "Darren Burns", "Scotland", 51.84),
]

class SortingStatus(Enum):
    UNSORTED = 0  #     [-] unsorted
    ASCENDING = 1  #    [↑] ascending (reverse = True)
    DESCENDING = 2  #   [↓] descending (reverse = False)


class TextualApp(App[None]):

    DEFAULT_CSS = """
    #datatable_container { align: center middle; }
    DataTable { width: 60; height: auto; }
    """

    sorting_statuses_dict: dict[str, SortingStatus] = {
        "lane": SortingStatus.UNSORTED,
        "swimmer": SortingStatus.UNSORTED, 
        "country": SortingStatus.UNSORTED, 
        "time": SortingStatus.UNSORTED,
    }

    def compose(self) -> ComposeResult:
        self.datatable = DataTable[Any]()
        with Container(id="datatable_container"):
            yield self.datatable

    def on_mount(self) -> None:
        
        self.datatable.add_column("lane [yellow]-[/]", key="lane")
        self.datatable.add_column("swimmer [yellow]-[/]", key="swimmer")
        self.datatable.add_column("country [yellow]-[/]", key="country")
        self.datatable.add_column("time [yellow]-[/]", key="time")
        self.datatable.add_rows(ROWS)

        # Retrieving the actual column from `DataTable.columns` is the trick
        # to making this work. This is different from `DataTable.get_column`, which
        # returns the values of the column, not the actual column object.
        # You need the column object to modify its label.  

        column = self.datatable.columns[ColumnKey("time")]
        self.sort_column(column, column.key)

    @on(DataTable.HeaderSelected)
    def header_selected(self, event: DataTable.HeaderSelected) -> None:
              
        column = self.datatable.columns[event.column_key]
        self.sort_column(column, column.key)
    
    def sort_column(self, column: Column, column_key: ColumnKey) -> None:

        if column_key.value not in self.sorting_statuses_dict:
            raise ValueError(
                f"Unknown column key: {column_key.value}. "
                "This should never happen, please report this issue."
            )

        key = column_key.value
        sort_status = self.sorting_statuses_dict
        table = self.datatable

        if sort_status[key] == SortingStatus.UNSORTED :
            # if column is unsorted, that means the user is switching which
            # column to sort. Start by resetting all columns to unsorted.
            for col_key in sort_status:
                sort_status[col_key] = SortingStatus.UNSORTED
                col_index = table.get_column_index(col_key)
                col = table.ordered_columns[col_index]
                col.label = Text.from_markup(f"{col_key} [yellow]-[/]")

            # Now set chosen column to ascending:
            sort_status[key] = SortingStatus.ASCENDING
            table.sort(column_key, reverse=True)
            column.label = Text.from_markup(f"{key} [yellow]↑[/]")

        # For the other two conditions, we just toggle ascending/descending
        elif sort_status[key] == SortingStatus.ASCENDING:
            sort_status[key] = SortingStatus.DESCENDING 
            table.sort(key, reverse=False)
            column.label = Text.from_markup(f"{key} [yellow]↓[/]")
        elif sort_status[key] == SortingStatus.DESCENDING: 
            sort_status[key] = SortingStatus.ASCENDING  
            table.sort(column_key, reverse=True)
            column.label = Text.from_markup(f"{key} [yellow]↑[/]")
        else:
            raise ValueError(
                f"Sort status for {key} is '{sort_status[key]}' "
                "did not meet any expected values."
            )
        

if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This script demonstrates how to modify the style of 
a Checkbox widget in the 'on' state.

Recipe by Edward Jazzhands"""

import sys
from textual.app import App
from textual.widgets import Static, Footer, Checkbox, Button
from textual.containers import Container

class TextualApp(App[None]):

    CSS = """
    Screen { align: center middle; }    
    #my_container { width: auto; height: auto; border: solid red;
    align: center middle; content-align: center middle; }
    Checkbox { 
        &.-on > .toggle--button {
            color: $text-success;
            background: green;
        }
    } 

    """
    
    def compose(self):

        with Container(id="my_container"):
            yield Static("Hello, Textual!")
            yield Checkbox()
            yield Button()

        yield Footer()


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This file demonstrates how to add the `control` method in a Textual message
to allow it to be specified in CSS selectors (query, @on, etc.).

Recipe by Edward Jazzhands"""


from __future__ import annotations
import sys
from textual import on
from textual.app import App
from textual.widgets import Static, Footer, Button
from textual.containers import Container
from textual.message import Message


class MyStatic(Static):

    class MessageFoo(Message):

        def __init__(self, sender: MyStatic):
            super().__init__()
            self.sender: MyStatic = sender

        @property
        def control(self) -> MyStatic:
            return self.sender

    def send_foo(self):
        self.post_message(self.MessageFoo(self))


class TextualApp(App[None]):

    DEFAULT_CSS = """
    Screen { align: center middle; }
    #my_static { border: solid blue; width: auto;}
    """

    def compose(self):

        with Container(id="my_container"):
            yield MyStatic("Hello, Textual!", id="my_static")
            yield Button("press me", id="my_button")

        yield Footer()

    def on_button_pressed(self):
        my_static = self.query_one(MyStatic)
        my_static.send_foo()

    @on(MyStatic.MessageFoo, "#my_static")
    def recieve_foo(self, message: MyStatic.MessageFoo):
        self.notify(f"Received MessageFoo from {message.sender}!")


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This demonstrates how to remove the focus highlight from buttons in Textual.
Note that the buttons are still focusable, but they will not show the
default focus highlight.

Recipe by Edward Jazzhands"""

import sys
from textual.app import App
from textual.widgets import Button
from textual.containers import Container

class TextualApp(App[None]):

    CSS = """
    #my_container { width: 1fr; height: 1fr; align: center middle; }
    .my_button {&:focus {text-style: none;}}
    """
    
    def compose(self):

        with Container(id="my_container"):
            yield Button("Button1", classes="my_button")
            yield Button("Button2", classes="my_button")
            yield Button("Button3", classes="my_button")


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This example demonstrates how to create a button with markup text.

Recipe by Edward Jazzhands"""

import sys
from textual.app import App
from textual.widgets import Footer, Button
from textual.containers import Container
from textual.content import Content

class TextualApp(App[None]):

    CSS = """
    #my_container { 
        width: 1fr; height: 1fr;
        border: solid red;
        align: center middle; content-align: center middle;
    }
    /* This just removes the focus highlight from the button text: */
    .buttons { &:focus {text-style: bold;} }
    """
    
    def compose(self):

        with Container(id="my_container"):
            button = Button(
                Content.from_markup("[yellow]This[/yellow] is my [red]button"),
                classes="buttons"
            )
            yield button
            yield Button("This is a normal button")
        yield Footer()


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This example demonstrates basic usage of colored text in a Static widget

Recipe by Edward Jazzhands"""

import sys
from textual.app import App
from textual.widgets import Static

class TextualApp(App[None]):

    CSS = """
    Screen { align: center middle; }
    #my_static {
        background: $surface;
        content-align: center middle;
        width: auto; height: 3;
        padding: 1;
    }
    """
    
    def compose(self):
        yield Static("[yellow]This[/yellow] is my [red]Static[/red]", id="my_static")
        # Note that Static has a `markup` argument which is True by default.


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This example demonstrates how to create a ListView with tooltips for each item.

Recipe by Edward Jazzhands"""

import sys
from textual.app import App, ComposeResult
from textual.widgets import Footer, Label, ListItem, ListView


class TextualApp(App[None]):
    
    CSS = """
    Screen {align: center middle;}
    ListView { width: 30; height: auto; }
    Label { padding: 1 2; }
    """

    def compose(self) -> ComposeResult:

        with ListView():
            item1 = ListItem(Label("One"))
            item1.tooltip = "This is the first item"
            item2 = ListItem(Label("Two"))
            item2.tooltip = "This is the second item"
            item3 = ListItem(Label("Three"))
            item3.tooltip = "This is the third item"

            yield item1
            yield item2
            yield item3

        yield Footer()


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This script demonstrates how the content of the MarkdownViewer widget
can be updated using the `document.update()` method.

Recipe by Edward Jazzhands"""

import sys
from textual import on
from textual.app import App, ComposeResult
from textual.widgets import MarkdownViewer, Button

MARKDOWN1 = """\
# Markdown Viewer

Markdown syntax and extensions are supported.

- Typography *emphasis*, **strong**, `inline code` etc.
- Headers
- Lists (bullet and ordered)
- Syntax highlighted code blocks
- Tables!

# Header 2
"""

MARKDOWN2 = """\
# Header 1

| Name            | Type   | Default | Description                        |
| --------------- | ------ | ------- | ---------------------------------- |
| `show_header`   | `bool` | `True`  | Show the table header              |
| `fixed_rows`    | `int`  | `0`     | Number of fixed rows               |
| `fixed_columns` | `int`  | `0`     | Number of fixed columns            |
| `zebra_stripes` | `bool` | `False` | Display alternating colors on rows |
| `header_height` | `int`  | `1`     | Height of header row               |
| `show_cursor`   | `bool` | `True`  | Show a cell cursor                 |

# Header 2
"""

class TextualApp(App[None]):

    def compose(self) -> ComposeResult:

        self.markdown_viewer = MarkdownViewer(MARKDOWN1, id="markdown_viewer")
        yield self.markdown_viewer
        yield Button("Markdown 1", id="markdown1")
        yield Button("Markdown 2", id="markdown2")

    @on(Button.Pressed, selector="#markdown1")
    def markdown1_pressed(self) -> None:
        self.markdown_viewer.document.update(MARKDOWN1)

    @on(Button.Pressed, selector="#markdown2")
    def markdown2_pressed(self) -> None:
        self.markdown_viewer.document.update(MARKDOWN2)        


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This file demonstrates how to animate a single widget's offset
using Textual's animation capabilities. The widget will move horizontally
when the button is pressed, and the animation will last for 1 second.

Recipe by Edward Jazzhands"""

import sys
from textual.app import App, ComposeResult
from textual.widgets import Static, Button
from textual.geometry import Offset

class TextualApp(App[None]):
    
    def compose(self) -> ComposeResult:
        yield Button("Animate Offset", id="animate_button")        
        self.box = Static("Hello, World!")
        yield self.box

    def on_button_pressed(self) -> None:
        self.box.animate("offset", value=Offset(20, 0), duration=1.0)


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This file demonstrates how to use on_click to get the widget
that was clicked within a Container in Textual.

Recipe by Edward Jazzhands"""

import sys
from textual.containers import Container
from textual.widgets import Static
from textual.app import App
from textual import events

class PopularContainer(Container):

    def on_click(self, event: events.Click) -> None:

        self.notify(f"card: {self.id}")
        # card = event.control      # <-- this would also work.
        # if card:
        #     self.notify(f"card: {card.id}")

class TextualApp(App[None]):

    DEFAULT_CSS = """
    Screen { align: center middle; }
    #my_container { border: solid blue; width: 50%; height: 50%;}
    #popular_container { border: solid red; width: 50%;}
    """

    def compose(self):

        with Container(id="my_container"):
            with PopularContainer(id="popular_container"):
                yield Static("Hello, Textual!", id="my_static")


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This script demonstrates how you could set redacted text and then reveal
the text when hovered.
The RedactedText widget has a reveal_range reactive attribute which is updated
when some redacted text is hovered. This will prompt a "smart refresh" to update
which ranges have the 'redacted' styling.
The important part is the metadata (meta) applied to portions of the text.
The mouse event handler checks the event.style.meta and updates the reveal_range
reactive attribute accordingly.

Recipe by Tom J Gooding"""

# These two imports added by Edward Jazzhands:
from __future__ import annotations    # added for 3.9 compatibility
import sys                            # added to conform to the recipe format

from rich.text import Text
from textual import events
from textual.app import App, ComposeResult, RenderResult
from textual.reactive import reactive
from textual.widget import Widget


class RedactedText(Widget):
    COMPONENT_CLASSES = {"redacted"}

    DEFAULT_CSS = """
    RedactedText {
        width: auto;
        height: auto;

        .redacted {
            color: $foreground;
            background: $foreground;
        }
    }
    """

    reveal_range: reactive[tuple[int, int] | None] = reactive(None)

    def render(self) -> RenderResult:
        redacted_style = self.get_component_rich_style("redacted")

        redacted_text = Text("Hello world foo bar")
        redact_ranges = [(6, 11), (16, 19)]

        for redact_range in redact_ranges:
            start, end = redact_range
            # Add meta info - see the mouse handler below
            redacted_text.apply_meta({"redacted": redact_range}, start, end)

            if self.reveal_range != redact_range:
                # Add a redacted style to portions of the text
                redacted_text.stylize(redacted_style, start, end)

        return redacted_text

    def on_mouse_move(self, event: events.MouseMove) -> None:
        meta = event.style.meta
        if "redacted" in meta:
            self.reveal_range = meta["redacted"]
        else:
            self.reveal_range = None

    def on_leave(self) -> None:
        self.reveal_range = None


class TextualApp(App):
    CSS = """
    Screen {
        align: center middle;
    }
    """

    def compose(self) -> ComposeResult:
        yield RedactedText()


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This script demonstrates how to create a Textual widget that is also
an abstract base class (ABC) using a metaclass that combines
the functionality of both `Widget` and `ABC`. This allows you to define
abstract methods that must be implemented by any subclass of the widget.
Note that I would not recommend doing this in general. It causes too
many issues with the Textual framework, such as preventing workers
from functioning correctly, and it can lead to unexpected behavior.
But its useful to keep this example up just as a demonstration of
how one would do it.

Recipe by Edward Jazzhands"""

import sys
from abc import ABC, abstractmethod
from textual.widget import Widget
from textual.app import App, ComposeResult


class ABCWidgetMeta(type(Widget), type(ABC)):
    pass


class AbstractDataWidget(Widget, ABC, metaclass=ABCWidgetMeta):
    
    @abstractmethod
    def load_data(self) -> None:
        pass
    
    def on_mount(self) -> None:
        self.load_data()


class UserListWidget(AbstractDataWidget):
    "Widget inheriting from AbstractDataWidget"
    
    # comment this method out to see the ABC error
    def load_data(self) -> None:
        self.notify("Data loaded from source")


class TextualApp(App[None]):

    def compose(self) -> ComposeResult:
        yield UserListWidget()


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This script demonstrates how to replace a widget's default "loading"
animation with a custom spinner widget in Textual.
The Spinner is using the rich.spinner module. You can replace the loading
animation with any Textual widget. The SpinnerWidget is only one example.

Recipe by Edward Jazzhands"""


from __future__ import annotations
import sys
import asyncio
from typing import Any, TYPE_CHECKING
if TYPE_CHECKING:
	from rich.console import RenderableType

from textual.app import App, ComposeResult
from rich.spinner import Spinner
from textual.widget import Widget
from textual.widgets import Button, Static

class SpinnerWidget(Static):

	DEFAULT_CSS = """
	SpinnerWidget {
		content-align: center middle;
		width: 1fr; height: 1fr;
	}
	"""

	def __init__(self, spinner: str, text: RenderableType = "", *args: Any, **kwargs: Any):

		super().__init__(*args, **kwargs)
		self._spinner = Spinner(spinner, text)  

	def on_mount(self) -> None:
		self.update_render = self.set_interval(1 / 60, self.update_spinner)

	def update_spinner(self) -> None:
		self.update(self._spinner)


class CustomLoadingWidget(Static):

	def render(self):
		return "This is the normal / finished state."	

	def get_loading_widget(self) -> Widget:
		return SpinnerWidget("bouncingBall", "Loading...")	
		
	async def loading_progress(self):
		
		self.loading = True
		for _ in range(4):
			await asyncio.sleep(0.75)
		self.loading = False


class TextualApp(App[None]):

	CSS = """
	Screen { align: center middle; }
	CustomLoadingWidget {
		border: solid red;
		width: 50%; height: 5;
		content-align: center middle;
	}
	"""
	
	def compose(self) -> ComposeResult:
		yield CustomLoadingWidget()
		yield Button("Start", id="start_button")

	async def on_button_pressed(self):
		my_widget = self.query_one(CustomLoadingWidget)
		await my_widget.loading_progress()		



if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This script demonstrates how you cannot query for screens directly using `query_one` or \
`query_all`. Instead, you should use the `get_screen` method to retrieve the screen instance. \
Attempting to query for the screen as shown below will cause the app to crash. \
Also note that the `get_default_screen` method is not needed if you are using \
`push_screen` to intialize with an installed screen (in SCREENS). This can cause issues \
because the default MainScreen instance will not be the same as the \
sinstalled screen in the app's screen stack.\

Example by Edward Jazzhands, 2025"""

from textual.app import App, ComposeResult
from textual.widgets import Static, Button
from textual.screen import Screen

class MainScreen(Screen[None]):

    def compose(self) -> ComposeResult:
        yield Static("Main Screen Content", id="main_static")
        yield Button("Press Me", id="main_button")

    def on_button_pressed(self, event: Button.Pressed) -> None:
        self.app.on_path_entered()

class TuiApp(App[None]):

    SCREENS = {"main": MainScreen}

    # def get_default_screen(self):    # This will cause problems. It is not needed.
    #     return MainScreen()          # if you use this instead of the push_screen method, you will see
    #                                  # that main_screen is not the same as self.screen.
    def on_mount(self) -> None:

        self.push_screen("main")
        self.log(self.screen_stack)

    def on_path_entered(self) -> None:

        # main_screen = self.query_one(MainScreen)     # this will not work. 

        main_screen = self.get_screen("main")    # <-- Do this instead
        self.notify(f"{main_screen is self.screen}")   # This shows that main_screen is the same as self.screen 

        main_screen.query_one("#main_static", Static).update("Path Entered!")


if __name__ == "__main__":
    app = TuiApp()
    app.run()


"""This demonstrates how to change the timezone of the header clock in Textual.
This is performing a monkey patch.

Recipe by Edward Jazzhands"""

import sys
from zoneinfo import ZoneInfo
from datetime import datetime

from textual.app import App, RenderResult
from textual.widgets import  Footer, Header
from textual.widgets._header import HeaderClock
from rich.text import Text

def render(self) -> RenderResult:
    """Render the header clock.

    Returns:
        The rendered clock.
    """
    return Text(datetime.now(ZoneInfo("UTC")).time().strftime(self.time_format))

HeaderClock.render = render

class TextualApp(App[None]):
    
    def compose(self):

        yield Header(show_clock=True)
        yield Footer()


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This file demonstrates how to create a Textual application
with a transparent background which can be toggled on and off.

Recipe by Edward Jazzhands"""

import sys
from textual import on
from textual.app import App
from textual.screen import Screen
from textual.widgets import Static, Footer, Button
from textual.containers import Container


class DummyScreen(Screen[None]):
    
    def on_mount(self) -> None:
        self.dismiss()


class TextualApp(App[None]):

    CSS = """
    Screen { align: center middle; }
    #my_static { border: solid blue; width: auto;}
    #main_container { width: 50%; height: 50%; border: solid red;
        align: center middle; }
    """

    def compose(self):

        with Container(id="main_container"):
            yield Static("Hello, Textual", id="my_static")
            yield Button("Toggle Transparency", id="toggle_transparency")

        yield Footer()

    @on(Button.Pressed, "#toggle_transparency")
    def toggle_transparency(self) -> None:

        self.ansi_color = not self.ansi_color
        self.push_screen(DummyScreen())


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This shows how to add a Submitted action to a TextArea widget which
replicates the Submitted messge in the Input widget, and which can
be triggered by pressing a key or combo (ctrl+s in this case).

Recipe by Edward Jazzhands"""

from __future__ import annotations
import sys
from textual import on
from textual.app import App
from textual.message import Message
from textual.binding import Binding
from textual.widgets import TextArea, Footer

class MyTextArea(TextArea):

    class Submitted(Message):
        def __init__(self, textarea: MyTextArea, text: str):
            super().__init__()
            self.textarea = textarea  
            self.text = text
    
    BINDINGS = [
        Binding("ctrl+s", "submit", "Submit", show=True),
    ]
    
    def action_submit(self):
        self.post_message(self.Submitted(self, self.text))



class TextualApp(App[None]):

    CSS = """
    Screen { align: center middle; }
    MyTextArea { border: solid blue; width: 50%; height: 50%; }
    """

    def compose(self):
        yield MyTextArea()
        yield Footer()
    
    @on(MyTextArea.Submitted)
    def my_textarea_submitted(self, event: MyTextArea.Submitted):
        self.notify(f"TextArea submitted with text: {event.text}")
        # handle submit here


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This file is a fairly complex example that shows off how Textual Screens
are handled by the App class and the DOM. It has 3 screens, which
all inherit from MyScreenType just to reduce code duplication (They are
all identical except for the text they display).
The main screen has a RichLog widget that displays debug information
when the user presses the "Print Debug" button (or the 'p' key).
The screens are transparent Modal screens and they also have a button to
print the debug information to the RichLog. You can see the info
being printed to the RichLog on the main screen below (although the Modal
Screen is the active screen on top).
You will see that for the `self.children` attribute on the App class,
there will only ever be one child, which is the currently active screen.
The docstring for `self.children` says this:

  > A view onto the app's immediate children.
  > This attribute exists on all widgets. In the case of the App, it will only ever
  > contain a single child, which will be the currently active screen.

You can see that when using the `self.query_children()` method, the currently
active screen will NOT be included in the results, even though it is technically
the only child of the App class (as shown by `self.children`).
Textual (I believe) automatically makes queries on the main App class
start the query on the currently active screen, as opposed to including it
in the query results.

Recipe by Edward Jazzhands"""


import sys
from textual import on
from textual.app import App, ComposeResult
from textual.screen import ModalScreen
from textual.binding import Binding
from textual.containers import Container, Horizontal
from textual.widgets import Static, Button, Footer, RichLog


class MyScreenType(ModalScreen[None]):

    def compose(self) -> ComposeResult:
        with Container(classes="screen_container"):
            yield Button("Print Debug", id="print_debug_button")
            yield Button("Close", id="close_button")
        yield Footer()

    @on(Button.Pressed, "#close_button")
    def close_button(self) -> None:
        self.dismiss()

    @on(Button.Pressed, "#print_debug_button")
    async def print_debug(self) -> None:
        await self.app.run_action("print_debug")
        

class ScreenOne(MyScreenType):

    def on_mount(self) -> None:
        self.query_one(Container).mount(
            Static("This is Screen One", classes="statics"), before=0
        )

class ScreenTwo(MyScreenType):

    def on_mount(self) -> None:
        self.query_one(Container).mount(
            Static("This is Screen Two", classes="statics"), before=0
        )


class ScreenThree(MyScreenType):

    def on_mount(self) -> None:
        self.query_one(Container).mount(
            Static("This is Screen Three", classes="statics"), before=0
        )

class TextualApp(App[None]):

    CSS = """
    ModalScreen { 
        align: center middle; 
        background: $background 30%;  /* extra transparent for demo */
    }
    .statics { padding: 1; }
    .left_container { align: center middle; width: 30; }
    .screen_container { border: solid $primary; width: auto; height: auto;}
    """

    BINDINGS = [
        Binding("up", "select('up')", description="Selection up", priority=True),
        Binding("down", "select('down')", description="Selection down", priority=True),
        Binding("p", "print_debug", description="Print debug information"),
    ]    

    # Screens here count as installed screens
    SCREENS = {
        "screen_one": ScreenOne,
        "screen_two": ScreenTwo,
        "screen_three": ScreenThree,
    }

    def compose(self) -> ComposeResult:

        with Horizontal():
            with Container(classes="left_container"):
                yield Button("Go to Screen One", id="screen_one_button", classes="main_button")
                yield Button("Go to Screen Two", id="screen_two_button", classes="main_button")
                yield Button("Go to Screen Three", id="screen_three_button", classes="main_button")
                yield Button("Print Debug", id="print_debug_button", classes="main_button")
            with Container(classes="right_container"):
                richlog = RichLog()
                richlog.can_focus = False
                yield richlog
        yield Footer()

    @on(Button.Pressed, ".main_button")
    def main_button_pressed(self, event: Button.Pressed) -> None:
        button_id = event.button.id
        if button_id == "print_debug_button":
            self.action_print_debug()
            return
        if button_id is not None:
            screen_name = button_id.replace("_button", "")
            self.query_one(RichLog).write(f"Opening screen: {screen_name}")
            self.push_screen(screen_name)

    def action_select(self, direction: str) -> None:
        """Handle up and down selection."""
        if direction == "up":
            self.action_focus_previous()
        else: 
            assert direction == "down"
            self.action_focus_next()

    def action_print_debug(self) -> None:
        """Print debug information to the RichLog."""
        log = self.query_one(RichLog)
        log.write("Debug Information:")
        log.write(f"Current Screen: {self.screen}")
        log.write(f"Screen Stack: {self.screen_stack}")
        log.write(f"Direct children of App: {self.children}")
        my_query = self.query_children().results()
        for item in my_query:
            log.write(f"Query Result: {item} (Type: {type(item)})")


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)


"""This script demonstrates how to use click links.

Recipe by Edward Jazzhands"""

import sys
from textual.app import App, ComposeResult
from textual.widgets import Label

class TextualApp(App[None]):

    BINDINGS = [
        ("f2", "show_note", "Show Notification"),
    ]

    def compose(self) -> ComposeResult:
        yield Label("Play the [on green @click=app.bell]bells[/] today!")
        yield Label("Show a [@click=app.show_note]notification[/]!")

    def action_show_note(self) -> None:
        self.notify("Action activated!")


if __name__ == "__main__":
    app = TextualApp()
    app.run()
    sys.exit(app.return_code)